#ifndef EVENT_H
#define EVENT_H

#include "General.h"
#include "Collectable.h"
#include "Collection.h"
#include "List.h"

typedef void* CLIENTDATA;
typedef void (*EVENT_CALLBACK)(CLIENTDATA);

#include "MTCP_SERVER.h"

class EVENT_FD : public COLLECTABLE {
private:
  int fd;
  EVENT_CALLBACK callback;
  CLIENTDATA data;
public:
  EVENT_FD(int _FD, EVENT_CALLBACK _CB, CLIENTDATA _D) : fd(_FD), callback(_CB), data(_D) {}
  void CALL() { callback(data); }
  int FD() { return fd; }
};

#define ALARM_ONCE  0
#define ALARM_MANY  1
#define ALARM_CHECK 2

class ALARM : public COLLECTABLE {
private:
  unsigned long alarm_type;
  unsigned long timer_duration;
  struct timeval start;
  struct timeval end;
  EVENT_CALLBACK callback;
  CLIENTDATA data;
public:
  ALARM(unsigned long, unsigned long, EVENT_CALLBACK, CLIENTDATA);
  void Off();
  static void Once(CLIENTDATA);
  static void Many(CLIENTDATA);
  static void Check(CLIENTDATA);
  void Ring();
  void TimeMarkEnd() { TIME_GET(&start); }
  unsigned long TimeDiff() { return TIME_DIFF_ULONG(start,end); }
  unsigned long TimerDuration() { return timer_duration; }
};

class EVENT {
private:
  static EVENT_CALLBACK rmp_callback;
  static SLIST event_fds;
  static SLIST alarms;
public:
  static void LOOP();
  static void RMPEvent_CB(EVENT_CALLBACK _RMP_CB) { rmp_callback = _RMP_CB; }
  static void Add_FD_CB(int, EVENT_CALLBACK, CLIENTDATA);
  static void Remove_FD_CB(int);
  static ALARM* SetAlarmOnce(unsigned long, EVENT_CALLBACK, CLIENTDATA);
  static ALARM* SetAlarmMany(unsigned long, EVENT_CALLBACK, CLIENTDATA);
  static ALARM* SetAlarmCheck(unsigned long, EVENT_CALLBACK, CLIENTDATA);
  static void RemoveAlarm(ALARM*&);
};

#endif
