#include "General.h"

int
s_pipe(int fd[2]) {
#if _PORT_
  return( pipe(fd) );
#else /* SOLARIS, LINUX, ULTRIX */
  return( socketpair(AF_UNIX, SOCK_STREAM, 0, fd) );
#endif
}

int
readn(int fd, void* vptr, int n) {
  int nleft;
  int nread;
  char* ptr;
  
  ptr = (char*)vptr;
  nleft = n;
  while( nleft>0 ) {
    if ((nread = read(fd, ptr, nleft))<0) {
      return( nread ); /* ERROR */
    } else {
      if (nread == 0) break; /* EOF */
    }
    nleft -= nread;
    ptr += nread;
  }
  return( n-nleft );
}

int
writen(int fd, void* vptr, int n) {
  int nleft;
  int nwritten;
  const char* ptr;

  ptr = (const char*)vptr;
  nleft = n;
  while( nleft>0 ) {
    if ((nwritten = write(fd, ptr, nleft))<=0) {
      return( nwritten ); /* ERROR */
    }
    nleft -= nwritten;
    ptr += nwritten;
  }
  return( n );
}

void
set_fl(int fd, int flags) {
  int val;
  val = fcntl(fd, F_GETFL, 0);
  val |= flags;
  fcntl(fd, F_SETFL, val);
}

void
clr_fl(int fd, int flags) {
  int val;
  val = fcntl(fd, F_GETFL, 0);
  val &= ~flags;
  fcntl(fd, F_SETFL, val);
}

CALLBACK_INT
SIGNAL_RESTART(int signo, CALLBACK_INT func) {
  struct sigaction act, oact;
  
  act.sa_handler = func;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;
  if (signo == SIGALRM) {
#ifdef SA_INTERRUPT
    act.sa_flags |= SA_INTERRUPT; /* SunOS */
#endif
  } else {
#ifdef SA_RESTART
    act.sa_flags |= SA_RESTART; /* SVR4, 4.3+BSD */
#endif
  }
  if (sigaction(signo, &act, &oact) < 0) {
    return(SIG_ERR);
  }
  return( oact.sa_handler );
}

CALLBACK_INT
SIGNAL_INTERRUPT(int signo, CALLBACK_INT func) {
  struct sigaction act, oact;
  
  act.sa_handler = func;
  sigemptyset(&act.sa_mask);
  act.sa_flags = 0;
#ifdef SA_INTERRUPT
  act.sa_flags |= SA_INTERRUPT; /* SunOS */
#endif
  if (sigaction(signo, &act, &oact) < 0) {
    return(SIG_ERR);
  }
  return( oact.sa_handler );
}
