#include "IPC.h"

/* IPCM CLASS */

ostrstream* IPCM::s = (ostrstream*)0;

void
IPCM::Reset() {
  delete s;
  s = new ostrstream;
}

void
IPCM::Issue(int fd) {
  char* str = S().str();
  unsigned long msg_length = htonl(S().pcount());
  writen(fd, (char*)(&msg_length), 4);
  writen(fd, str, S().pcount());
  delete str;
}

istrstream*
IPCM::Accept(int fd) {
  unsigned long msg_length;
  if (readn(fd, (char*)(&msg_length), sizeof(unsigned long)) == 0) exit(0);
  msg_length = ntohl(msg_length);
  char* msg = new char[msg_length];
  if (readn(fd, msg, msg_length) == 0) exit(0);
  return new istrstream(msg,(int)msg_length);
}

/* IPC CLASS */

pid_t IPC::pid = 0;
int IPC::fd = 0;
unsigned long IPC::server_port = 0;
ostrstream* IPC::s = (ostrstream*)0;

void
IPC::Reset() {
  delete s;
  s = new ostrstream;
}

void
IPC::Issue() {
  char* str = S().str();
  unsigned long msg_length = htonl(S().pcount());
  writen(fd, (char*)(&msg_length), 4);
  writen(fd, str, S().pcount());
  delete str;
}

istrstream*
IPC::Accept() {
  unsigned long msg_length;
  if (readn(fd, (char*)(&msg_length), sizeof(unsigned long)) == 0) exit(0);
  msg_length = ntohl(msg_length);
  char* msg = new char[msg_length];
  if (readn(fd, msg, msg_length) == 0) exit(0);
  return new istrstream(msg,(int)msg_length);
}

void
IPC::Init(CALLBACK_INT server_callback) {
  // Create Stream Pipe.
  int tmpfd[2];
  if (s_pipe(tmpfd) < 0) {
    cerr << "Error creating pipe needed for MTCP Server IPC." << endl; cerr.flush();
    exit(1);
  }

  pid = fork();
  if (pid < 0) {
    cerr << "Error spawning MTCP Server process." << endl; cerr.flush();
    exit(1);
  }
  if (pid > 0) { // Parent. (Server)
    close(tmpfd[0]);
    server_callback(tmpfd[1]);
    exit(0);
  } else { // Child.
    close(tmpfd[1]);
    fd = tmpfd[0];
    Reset();
    S() << encode(fd);
    S() << encode((int)getpid());
    Issue();
    istrstream* msg = Accept();
    (*msg) >> decode(server_port);
    delete msg;
    /* Child Returns. */
  }
}

int
IPC::ObtainConnection(int type) {
  int sockid = socket(AF_INET, SOCK_STREAM, 0);
  if (sockid >= 0) {
    /* Connect. */
    struct sockaddr_in server;
    memset((char*)&server, (char)0, sizeof(server));
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
    server.sin_port = htons((unsigned short)server_port);
    if (connect(sockid, (struct sockaddr*)&server, sizeof(server)) == -1) {
      close(sockid);
      return -1; /* ERROR */
    }
    /* Initialize Connection. */
    IPCM::Reset();
    IPCM::S() << encode(type);
    IPCM::Issue(sockid);
    IPCM::Reset();
    IPCM::S() << encode(sockid);
    IPCM::S() << encode((int)getpid());
    IPCM::Issue(sockid);
    delete IPCM::Accept(sockid); /* Wait */
    if (type == CONNECTION_IPC) {
      close(fd);
      fd = sockid;
    }
  }
  return( sockid );
}
