#ifndef MTCP_SERVER_H
#define MTCP_SERVER_H

#include "General.h"
#include "List.h"
#include "Integer.h"
#include "Event.h"
#include "IPC.h"

class MTCP_SOCKET;

class MTCP_MESSAGE : public COLLECTABLE {
private:
  ostrstream* s;
  char* msg_data;
  unsigned long int msg_length;
public:
  MTCP_MESSAGE();
  ~MTCP_MESSAGE();
  ostrstream& S() { return (*s); }
  SendStatus Issue(MTCP_SOCKET*);
};

class MTCP_CONNECTION : public COLLECTABLE {
protected:
  int fd;
  int owner_fd;
  SLIST owner_pids;
  MTCP_CONNECTION() {}
public:
  MTCP_CONNECTION(int);
  ~MTCP_CONNECTION();
  int IDEQ(int, int);
  int PIDEQ(int);
  void AppendPID(int);
  /* Messaging API */
protected:
  ostrstream* s;
public:
  void Reset();
  ostrstream& S() { return (*s); }
  void Issue();
  istrstream* Accept();
};

class MTCP_SOCKET : public MTCP_CONNECTION {
protected:
  /* The messaging layer should be used here; however, it
     is not currently available, so just faking something
     will work for now.  */
  // mClient* mc;
  int member;
  SLIST message_queue;
  MTCP_MESSAGE* message;
  mClient client;
  mRMPGroup* group;
  unsigned long qos;
  int message_level;
  struct sockaddr_rmp rmpaddr;
  int number_accepts;
public:
  MTCP_SOCKET(int);
  ~MTCP_SOCKET();
  int Connect(istrstream&);
  int Close(int);
  void HandleData();
  void HandleMessage(char*, int);
  void Member(mRMPGroup*);
  void SendQ();
  void BeginMessage();
  void EndMessage();
  SendStatus Send(char*, unsigned long int);
  void GetSockName(ostream&);
  int SetSockOpt(istream&);
  void GetSockOpt(istream&,ostream&);
  void Accept(MTCP_CONNECTION*);
  void Bind();
};

class MTCP_SERVER {
private:
  static int fd;
  static SLIST ipc_connections;
  static SLIST socket_connections;
  static CALLBACK_INT signal_TSTP;
  static CALLBACK_INT signal_CONT;
  static int terminate;
  static int exit_status;
public:
  static void Init(int);
  static void Handle_SocketConnection(CLIENTDATA);
  static void Handle_IPCMessage(CLIENTDATA);
  static void Handle_SocketData(CLIENTDATA);
  static void Handle_RMP_Event(CLIENTDATA);
  static void InitSignalHandlers();
  static void ForwardSignal(int);
  static void SignalCHLD(int);
  static int TerminateQ();
};

#endif
